import _values from 'lodash/values';
import _get from 'lodash/get';
import _each from 'lodash/each';
import _keys from 'lodash/keys';
import _isEmpty from 'lodash/isEmpty';

var _class, _temp;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React from 'react';
import PropTypes from 'prop-types';

import { Children } from '@react-mapboxgl/core';
import Click from '@react-mapboxgl/click';

var Toggle = (_temp = _class = function (_React$Component) {
  _inherits(Toggle, _React$Component);

  function Toggle() {
    _classCallCheck(this, Toggle);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this));

    _this.state = {
      features: {}
    };

    _this.handleClickLayer = _this.handleClickLayer.bind(_this);
    _this.handleClickMap = _this.handleClickMap.bind(_this);
    _this.toggleFeature = _this.toggleFeature.bind(_this);
    return _this;
  }

  Toggle.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if (this.props.layer !== nextProps.layer) {
      this.setState({ features: {} });
    }
    if (this.props.multiple !== nextProps.multiple) {
      if (!nextProps.multiple && !_isEmpty(this.state.features)) {
        var _features;

        var key = _keys(this.state.features).pop();
        this.setState({ features: (_features = {}, _features[key] = this.state.features[key], _features) });
      }
    }
  };

  Toggle.prototype.handleClickLayer = function handleClickLayer(e) {
    if (this.props.multiple) {
      _each(e.features, this.toggleFeature);
    } else if (e.features.length) {
      this.toggleFeature(e.features[0]);
    }
  };

  Toggle.prototype.handleClickMap = function handleClickMap(e) {
    var map = this.context.map;

    var features = map.queryRenderedFeatures(e.point, {
      layers: [this.props.layer]
    });
    if (!features.length) {
      if (this.props.closeOnClickOutside) {
        this.setState({ features: {} });
      }
    } else if (this.props.multiple) {
      _each(features, this.toggleFeature);
    } else {
      this.toggleFeature(features[0]);
    }
  };

  Toggle.prototype.toggleFeature = function toggleFeature(feature) {
    var features = this.state.features;

    var property = this.getProperty(feature);
    var isOn = true;

    if (this.props.multiple) {
      if (this.state.features[property]) {
        isOn = false;
        delete features[property];
      } else {
        features[property] = feature;
      }
    } else {
      if (features[property]) {
        isOn = false;
        features = {};
      } else {
        var _features2;

        features = (_features2 = {}, _features2[property] = feature, _features2);
      }
    }

    if (this.props.onToggle) {
      this.props.onToggle(feature, isOn);
    }

    this.setState({ features: features });
  };

  Toggle.prototype.getProperty = function getProperty(feature) {
    var propertyPath = 'properties.' + this.props.property;
    var property = _get(feature, propertyPath);
    if (typeof property !== 'undefined') {
      return property;
    } else {
      throw new Error('Could not find property for <Toggle> feature');
    }
  };

  Toggle.prototype.render = function render() {
    return React.createElement(
      Children,
      null,
      this.props.closeOnClickOutside ? React.createElement(Click, {
        event: this.props.clickEvent,
        avoidDoubleClick: this.props.avoidDoubleClick,
        doubleClickSpeed: this.props.doubleClickSpeed,
        onClick: this.handleClickMap
      }) : React.createElement(Click, {
        layer: this.props.layer,
        event: this.props.clickEvent,
        avoidDoubleClick: this.props.avoidDoubleClick,
        doubleClickSpeed: this.props.doubleClickSpeed,
        onClick: this.handleClickLayer
      }),
      this.props.children ? typeof this.props.children === 'function' ? this.props.children({
        features: _values(this.state.features),
        properties: _keys(this.state.features)
      }) : this.props.children : null
    );
  };

  return Toggle;
}(React.Component), _class.propTypes = {
  layer: PropTypes.string.isRequired,
  property: PropTypes.string.isRequired,
  multiple: PropTypes.bool,
  clickEvent: PropTypes.string,
  avoidDoubleClick: PropTypes.bool,
  doubleClickSpeed: PropTypes.number,
  closeOnClickOutside: PropTypes.bool,
  onToggle: PropTypes.func,
  children: PropTypes.func
}, _class.defaultProps = {
  multiple: false,
  closeOnClickOutside: true
}, _class.contextTypes = {
  map: PropTypes.object
}, _temp);


export default Toggle;