'use strict';

exports.__esModule = true;

var _values2 = require('lodash/values');

var _values3 = _interopRequireDefault(_values2);

var _get2 = require('lodash/get');

var _get3 = _interopRequireDefault(_get2);

var _each2 = require('lodash/each');

var _each3 = _interopRequireDefault(_each2);

var _keys2 = require('lodash/keys');

var _keys3 = _interopRequireDefault(_keys2);

var _isEmpty2 = require('lodash/isEmpty');

var _isEmpty3 = _interopRequireDefault(_isEmpty2);

var _class, _temp;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _core = require('@react-mapboxgl/core');

var _click = require('@react-mapboxgl/click');

var _click2 = _interopRequireDefault(_click);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Toggle = (_temp = _class = function (_React$Component) {
  _inherits(Toggle, _React$Component);

  function Toggle() {
    _classCallCheck(this, Toggle);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this));

    _this.state = {
      features: {}
    };

    _this.handleClickLayer = _this.handleClickLayer.bind(_this);
    _this.handleClickMap = _this.handleClickMap.bind(_this);
    _this.toggleFeature = _this.toggleFeature.bind(_this);
    return _this;
  }

  Toggle.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if (this.props.layer !== nextProps.layer) {
      this.setState({ features: {} });
    }
    if (this.props.multiple !== nextProps.multiple) {
      if (!nextProps.multiple && !(0, _isEmpty3.default)(this.state.features)) {
        var _features;

        var key = (0, _keys3.default)(this.state.features).pop();
        this.setState({ features: (_features = {}, _features[key] = this.state.features[key], _features) });
      }
    }
  };

  Toggle.prototype.handleClickLayer = function handleClickLayer(e) {
    if (this.props.multiple) {
      (0, _each3.default)(e.features, this.toggleFeature);
    } else if (e.features.length) {
      this.toggleFeature(e.features[0]);
    }
  };

  Toggle.prototype.handleClickMap = function handleClickMap(e) {
    var map = this.context.map;

    var features = map.queryRenderedFeatures(e.point, {
      layers: [this.props.layer]
    });
    if (!features.length) {
      if (this.props.closeOnClickOutside) {
        this.setState({ features: {} });
      }
    } else if (this.props.multiple) {
      (0, _each3.default)(features, this.toggleFeature);
    } else {
      this.toggleFeature(features[0]);
    }
  };

  Toggle.prototype.toggleFeature = function toggleFeature(feature) {
    var features = this.state.features;

    var property = this.getProperty(feature);
    var isOn = true;

    if (this.props.multiple) {
      if (this.state.features[property]) {
        isOn = false;
        delete features[property];
      } else {
        features[property] = feature;
      }
    } else {
      if (features[property]) {
        isOn = false;
        features = {};
      } else {
        var _features2;

        features = (_features2 = {}, _features2[property] = feature, _features2);
      }
    }

    if (this.props.onToggle) {
      this.props.onToggle(feature, isOn);
    }

    this.setState({ features: features });
  };

  Toggle.prototype.getProperty = function getProperty(feature) {
    var propertyPath = 'properties.' + this.props.property;
    var property = (0, _get3.default)(feature, propertyPath);
    if (typeof property !== 'undefined') {
      return property;
    } else {
      throw new Error('Could not find property for <Toggle> feature');
    }
  };

  Toggle.prototype.render = function render() {
    return _react2.default.createElement(
      _core.Children,
      null,
      this.props.closeOnClickOutside ? _react2.default.createElement(_click2.default, {
        event: this.props.clickEvent,
        avoidDoubleClick: this.props.avoidDoubleClick,
        doubleClickSpeed: this.props.doubleClickSpeed,
        onClick: this.handleClickMap
      }) : _react2.default.createElement(_click2.default, {
        layer: this.props.layer,
        event: this.props.clickEvent,
        avoidDoubleClick: this.props.avoidDoubleClick,
        doubleClickSpeed: this.props.doubleClickSpeed,
        onClick: this.handleClickLayer
      }),
      this.props.children ? typeof this.props.children === 'function' ? this.props.children({
        features: (0, _values3.default)(this.state.features),
        properties: (0, _keys3.default)(this.state.features)
      }) : this.props.children : null
    );
  };

  return Toggle;
}(_react2.default.Component), _class.propTypes = {
  layer: _propTypes2.default.string.isRequired,
  property: _propTypes2.default.string.isRequired,
  multiple: _propTypes2.default.bool,
  clickEvent: _propTypes2.default.string,
  avoidDoubleClick: _propTypes2.default.bool,
  doubleClickSpeed: _propTypes2.default.number,
  closeOnClickOutside: _propTypes2.default.bool,
  onToggle: _propTypes2.default.func,
  children: _propTypes2.default.func
}, _class.defaultProps = {
  multiple: false,
  closeOnClickOutside: true
}, _class.contextTypes = {
  map: _propTypes2.default.object
}, _temp);
exports.default = Toggle;